#!/usr/bin/env python
#-*- coding: utf-8 -*-

import ossaudiodev
import audioop
import socket
import select
import thread
import time
import wave
import os

SRV_PORT = 3030
SRV_TRIGPORT = 3031
SRV_BUFFER = 1500
SND_DEVICE = "/dev/dsp"
SND_FMT = ossaudiodev.AFMT_S16_LE
SND_NCHAN = 1
SND_SRATE = 8000
SND_RECTIME = 5
SND_COUNT = -1
SND_INTERVAL = 30
FILE_DIR = "recordings"
FILE_DATE = "%j_%H%M%S_"
TRIGGER = "./pp"

state = {
'device' : SND_DEVICE,
'format' : SND_FMT,
'channels' : SND_NCHAN,
'samplerate' : SND_SRATE,
'rectime' : SND_RECTIME,
'count' : SND_COUNT,
'udptrig' : False,
'interval' : SND_INTERVAL,
'ready' : True,
'run' : True,
'filename' : '',
'fileno' : 0,
'shotno' : 0
}

def uclistate(cli):
	# Send current state to client
	cli.send("HELLO %s\n" % addr[0])
	cli.send("DEVICE %s\n" % state['device'])
	cli.send("SAMPLERATE %s\n" % state['samplerate'])
	cli.send("CHANNELS %s\n" % state['channels'])
	cli.send("RECTIME %i\n" % state['rectime'])
	cli.send("COUNT %i\n" % state['count'])
	cli.send("INTERVAL %i\n" % state['interval'])
	
	if state['udptrig']:
		cli.send("TRIGGER EXTERN\n")
	else:
		cli.send("TRIGGER INTERN\n")
		
	if state['ready']:
		cli.send("STOPPED\n")
	else:
		cli.send("RUNNING\n")
	
def broadcast(msg):
	for sfd in clients.keys():
		if sfd == srv.fileno():
			continue
			
		try:
			clients[sfd].send(msg)
		except socket.error:
			poll.unregister(sfd)
			clients[sfd].close()
			del clients[sfd]

def cmdexec(cmd, cli):
	words = cmd.split(" ")
	
	if words[0] == "SAMPLERATE":
		state['samplerate'] = dsp.speed(int(words[1]))
		broadcast("SAMPLERATE %s\n" % state['samplerate'])
		# print "Changed samplerate to %s by request from %s " % (state['samplerate'], cli.getpeername()[0])

	elif words[0] == "RECTIME":
		state['rectime'] = int(words[1])
		broadcast("RECTIME %s\n" % state['rectime'])
		
	elif words[0] == "COUNT":
		state['count'] = int(words[1])
		broadcast("COUNT %s\n" % state['count'])
		
	elif words[0] == "INTERVAL":
		state['interval'] = int(words[1])
		broadcast("INTERVAL %s\n" % state['interval'])
		
	elif words[0] == "TRIGGER":
		if words[1] == "EXTERN":
			state['udptrig'] = True
			broadcast("TRIGGER EXTERN\n")
		else:
			state['udptrig'] = False
			broadcast("TRIGGER INTERN\n")
			
	elif words[0] == "RUN":
		state['ready'] = False
		broadcast("RUNNING")
		thread.start_new_thread(recthread, ())

	elif words[0] == "STOP":
		state['ready'] = True

def statsthread():
	while state['ready']:
		samplebuffer = dsp.read(state['samplerate'])
		
		for sfd in clients.keys():
			if sfd == srv.fileno():
				continue
			
			try:
				clients[sfd].send("STATS %i %i %i\n" % (audioop.cross(samplebuffer, 2), \
				((audioop.maxpp(samplebuffer, 2) * 100) / 32768), \
				((audioop.rms(samplebuffer, 2) * 100) / 32768)))
			except socket.error:
				poll.unregister(sfd)
				clients[sfd].close()
				del clients[sfd]
				
def recthread():
	state['shotno'] += 1
	state['filename'] = "%s%i.wav" % (time.strftime(FILE_DATE), state['shotno'])
	recnum = 0
	recfp = wave.open("%s/%s" % (FILE_DIR, state['filename']), "wb")
	
	broadcast("FILENAME %s\n" % state['filename'])
	
	recfp.setnchannels(state['channels'])
	recfp.setsampwidth(2)
	recfp.setframerate(state['samplerate'])
	
	# Open trigger pipe
	trig = os.popen(TRIGGER, "w")
	
	while True:
		if len(dsp.read(2)):
			break
	
	broadcast("STATS - - -\n")
	
	if state['udptrig']:
		while not state['ready']:
			ts = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			ts.bind(('', SRV_TRIGPORT))

			tp = select.poll()
			tp.register(ts)

			while not state['ready']:
				event = tp.poll(1000)

				if event[0] == ts.fileno():
					eiva = ts.recvfrom(SRV_BUFFER)

					recnum += 1
					broadcast("RECNUM %i\nTIMELEFT S\n" % recnum)

					trig.write("T\n")
					trig.flush()

					samplebuffer = dsp.read(state['samplerate'] * state['rectime'] * 2)
					recfp.writeframes(samplebuffer)

					broadcast("TIMELEFT -\n")

					if recnum == state['count']:
						state['ready'] = True

			ts.close()
	else:
		while not state['ready']:
			timeleft = state['interval']
			
			while timeleft and not state['ready']:
				timeleft -= 1
				time.sleep(1)
				broadcast("TIMELEFT %i\n" % timeleft)
				
			if state['ready']:
				break
			
			recnum += 1
			broadcast("RECNUM %i\nTIMELEFT S\n" % recnum)
			
			trig.write("T\n")
			trig.flush()
			
			samplebuffer = dsp.read(state['samplerate'] * state['rectime'] * 2)
			recfp.writeframes(samplebuffer)

			if recnum == state['count']:
				state['ready'] = True
			
	
	broadcast("TIMELEFT -\nSTOPPED\n")
	thread.start_new_thread(statsthread, ())
	
	recfp.close()
	
	trig.write("C\n")
	trig.flush()
	trig.close()
	
# Open soundcard and set initial format, samplerate and channels
print "Initializing audio device: %s" % state['device']

dsp = ossaudiodev.open(state['device'], 'r')
dsp.setparameters(state['format'], state['channels'], state['samplerate']) 
state['samplerate'] = dsp.speed(state['samplerate'])

print "Initial setup: %i channel(s), %i Hz samplerate" % (state['channels'], state['samplerate'])

# Create listening socket
srv = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
srv.bind(("", SRV_PORT))
srv.listen(1)

print "Listening on port %i..." % SRV_PORT

# Set up select object and register listening socket
poll = select.poll()
poll.register(srv, select.POLLIN | select.POLLPRI)

# Dictionary with currently connected clients
clients = {}

# Monitor soundcard input and send statistics to connected clients
thread.start_new_thread(statsthread, ())

# Wait for connections or data
while state['run']:
	waiting = poll.poll(1000)
	
	for item in waiting:
		# Check if a new client is in the queue
		if item[0] == srv.fileno():
			cli, addr = srv.accept()
			clients[cli.fileno()] = cli
			cli.setblocking(0)
			
			poll.register(cli, select.POLLIN | select.POLLERR | select.POLLHUP | select.POLLNVAL)
			
			print "New connection from %s:%i" % (addr[0], addr[1])
			
			# Enlighten client
			uclistate(cli)
			
		elif item[1] == select.POLLIN:
			cli = clients[item[0]]
			data = cli.recv(SRV_BUFFER)
			
			if len(data) == 0:
				print "Closing connection to %s:%i" % (cli.getpeername()[0], cli.getpeername()[1])
				poll.unregister(cli)
				del clients[item[0]]
				cli.close()
				
			else:
				print "Got data from %s: %s" % (cli.getpeername()[0], data.strip())
				
				if data.strip() == "SHUTDOWN":
					broadcast("SHUTDOWN")
					cli = clients[item[0]]
					del clients[item[0]]
					poll.unregister(cli)
					cli.close()
					state['run'] = False
					break;
					
				cmdexec(data.strip(), clients[item[0]])
		
		else:
			# Error on client socket, close connection
			cli = clients[item[0]]
			del clients[item[0]]
			poll.unregister(cli)
			# print "Lost connection to %s:%i " % (cli.getpeername()[0], cli.getpeername()[1])
			cli.close()
						
print "Shutting down"

srv.close()
