#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <strings.h>
#include <termios.h>

#define PORT 3000
#define TERM "/dev/tts/1"
#define BAUDRATE B9600
#define BUFFER 1408

int main(int argc, char *argv[]);
int dev_open(char *device, int baudrate);
int dev_close(int dev);

int main(int argc, char *argv[])
{
	int dev;
	struct sockaddr_in sin;
	struct sockaddr_in pin;
	int serverd;
	int clientd;
	int size;
	int ret;
	char buff[BUFFER];

	serverd = socket(AF_INET, SOCK_STREAM, 0);

	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(PORT);
	
	bind(serverd, (struct sockaddr *)&sin, sizeof(sin));

	listen(serverd, 2);

	for (;;)
	{
		clientd = accept(serverd, (struct sockaddr *)&pin, &size);

		dev = dev_open(TERM, BAUDRATE);

		for (;;)
		{
			ret = read(dev, buff, BUFFER);

			if (ret <= 0)
				break;

			ret = send(clientd, buff, ret, 0);

			if (ret == -1)
				break;
		}

		dev_close(dev);

		close(clientd);

	}

	return 0;
}

int dev_open(char *device, int baudrate)
{
	int dev; /* Serial port file descriptor */
	struct termios termopt;

	/* Open serial port */
	dev = open(device, O_RDONLY|O_NOCTTY);

	if (dev == -1)
		return dev;

	/* Clear terminal options structure */
	bzero(&termopt, sizeof(termopt));

	/* Set baudrate, 8n1, ignore modem control lines and activate receiver */
	termopt.c_cflag = baudrate | CS8 | CLOCAL | CREAD;

	/* Ignore parity errors */
	termopt.c_iflag = IGNPAR;

	termopt.c_cc[VMIN] = 1;
	termopt.c_cc[VTIME] = 0;

	/* Flush data received but not read and save settings */
	tcflush(dev, TCIFLUSH);
	tcsetattr(dev, TCSANOW, &termopt);

	return dev;
}

int dev_close(int dev)
{
	int ret;

	ret = close(dev);

	return ret;
}
